@@ -93,6 +93,60 @@ polkit_backend_authority_system_bus_name_owner_changed (PolkitBackendAuthority
     klass->system_bus_name_owner_changed (authority, name, old_owner, new_owner);
 }
 
+/**
+ * polkit_backend_authority_get_name:
+ * @authority: A #PolkitBackendAuthority.
+ *
+ * Gets the name of the authority backend.
+ *
+ * Returns: The name of the backend.
+ */
+const gchar *
+polkit_backend_authority_get_name (PolkitBackendAuthority *authority)
+{
+  PolkitBackendAuthorityClass *klass;
+  klass = POLKIT_BACKEND_AUTHORITY_GET_CLASS (authority);
+  if (klass->get_name == NULL)
+    return "(not set)";
+  return klass->get_name (authority);
+}
+
+/**
+ * polkit_backend_authority_get_version:
+ * @authority: A #PolkitBackendAuthority.
+ *
+ * Gets the version of the authority backend.
+ *
+ * Returns: The name of the backend.
+ */
+const gchar *
+polkit_backend_authority_get_version (PolkitBackendAuthority *authority)
+{
+  PolkitBackendAuthorityClass *klass;
+  klass = POLKIT_BACKEND_AUTHORITY_GET_CLASS (authority);
+  if (klass->get_version == NULL)
+    return "(not set)";
+  return klass->get_version (authority);
+}
+
+/**
+ * polkit_backend_authority_get_features:
+ * @authority: A #PolkitBackendAuthority.
+ *
+ * Gets the features supported by the authority backend.
+ *
+ * Returns: Flags from #PolkitAuthorityFeatures.
+ */
+PolkitAuthorityFeatures
+polkit_backend_authority_get_features (PolkitBackendAuthority *authority)
+{
+  PolkitBackendAuthorityClass *klass;
+  klass = POLKIT_BACKEND_AUTHORITY_GET_CLASS (authority);
+  if (klass->get_features == NULL)
+    return POLKIT_AUTHORITY_FEATURES_NONE;
+  return klass->get_features (authority);
+}
+
 /**
  * polkit_backend_authority_enumerate_actions:
  * @authority: A #PolkitBackendAuthority.
@@ -416,7 +470,7 @@ polkit_backend_authority_revoke_temporary_authorizations (PolkitBackendAuthority
  *
  * Revokes a temporary authorizations with opaque identifier @id.
  *
- * Returns: %TRUE if the operatoin succeeded, %FALSE if @error is set.
+ * Returns: %TRUE if the operation succeeded, %FALSE if @error is set.
  **/
 gboolean
 polkit_backend_authority_revoke_temporary_authorization_by_id (PolkitBackendAuthority   *authority,
@@ -442,6 +496,153 @@ polkit_backend_authority_revoke_temporary_authorization_by_id (PolkitBackendAuth
     }
 }
 
+/**
+ * polkit_backend_authority_add_lockdown_for_action:
+ * @authority: A #PolkitBackendAuthority.
+ * @caller: The system bus name that called the method.
+ * @action_id: The action id.
+ * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
+ * @user_data: The data to pass to @callback.
+ *
+ * Asynchronously add locks down for @action_id.
+ *
+ * When the operation is finished, @callback will be invoked. You can
+ * then call polkit_backend_authority_add_lockdown_for_action_finish()
+ * to get the result of the operation.
+ */
+void
+polkit_backend_authority_add_lockdown_for_action (PolkitBackendAuthority  *authority,
+                                                  PolkitSubject           *caller,
+                                                  const gchar             *action_id,
+                                                  GAsyncReadyCallback      callback,
+                                                  gpointer                 user_data)
+{
+  PolkitBackendAuthorityClass *klass;
+
+  klass = POLKIT_BACKEND_AUTHORITY_GET_CLASS (authority);
+
+  if (klass->add_lockdown_for_action == NULL)
+    {
+      GSimpleAsyncResult *simple;
+
+      simple = g_simple_async_result_new_error (G_OBJECT (authority),
+                                                callback,
+                                                user_data,
+                                                POLKIT_ERROR,
+                                                POLKIT_ERROR_NOT_SUPPORTED,
+                                                "Operation not supported");
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
+    }
+  else
+    {
+      klass->add_lockdown_for_action (authority, caller, action_id, callback, user_data);
+    }
+}
+
+/**
+ * polkit_backend_authority_add_lockdown_for_action_finish:
+ * @authority: A #PolkitBackendAuthority.
+ * @res: A #GAsyncResult obtained from the callback.
+ * @error: Return location for error or %NULL.
+ *
+ * Finishes adding lock down for an action.
+ *
+ * Returns: %TRUE if the operation succeeded or, %FALE if @error is set.
+ */
+gboolean
+polkit_backend_authority_add_lockdown_for_action_finish (PolkitBackendAuthority  *authority,
+                                                         GAsyncResult            *res,
+                                                         GError                 **error)
+{
+  PolkitBackendAuthorityClass *klass;
+
+  klass = POLKIT_BACKEND_AUTHORITY_GET_CLASS (authority);
+
+  if (klass->add_lockdown_for_action_finish == NULL)
+    {
+      g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+      return FALSE;
+    }
+  else
+    {
+      return klass->add_lockdown_for_action_finish (authority, res, error);
+    }
+}
+
+/**
+ * polkit_backend_authority_remove_lockdown_for_action:
+ * @authority: A #PolkitBackendAuthority.
+ * @caller: The system bus name that called the method.
+ * @action_id: The action id.
+ * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
+ * @user_data: The data to pass to @callback.
+ *
+ * Asynchronously remove locks down for @action_id.
+ *
+ * When the operation is finished, @callback will be invoked. You can
+ * then call polkit_backend_authority_remove_lockdown_for_action_finish()
+ * to get the result of the operation.
+ */
+void
+polkit_backend_authority_remove_lockdown_for_action (PolkitBackendAuthority  *authority,
+                                                     PolkitSubject           *caller,
+                                                     const gchar             *action_id,
+                                                     GAsyncReadyCallback      callback,
+                                                     gpointer                 user_data)
+{
+  PolkitBackendAuthorityClass *klass;
+
+  klass = POLKIT_BACKEND_AUTHORITY_GET_CLASS (authority);
+
+  if (klass->remove_lockdown_for_action == NULL)
+    {
+      GSimpleAsyncResult *simple;
+
+      simple = g_simple_async_result_new_error (G_OBJECT (authority),
+                                                callback,
+                                                user_data,
+                                                POLKIT_ERROR,
+                                                POLKIT_ERROR_NOT_SUPPORTED,
+                                                "Operation not supported");
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
+    }
+  else
+    {
+      klass->remove_lockdown_for_action (authority, caller, action_id, callback, user_data);
+    }
+}
+
+/**
+ * polkit_backend_authority_remove_lockdown_for_action_finish:
+ * @authority: A #PolkitBackendAuthority.
+ * @res: A #GAsyncResult obtained from the callback.
+ * @error: Return location for error or %NULL.
+ *
+ * Finishes removing lock down for an action.
+ *
+ * Returns: %TRUE if the operation succeeded or, %FALE if @error is set.
+ */
+gboolean
+polkit_backend_authority_remove_lockdown_for_action_finish (PolkitBackendAuthority  *authority,
+                                                            GAsyncResult            *res,
+                                                            GError                 **error)
+{
+  PolkitBackendAuthorityClass *klass;
+
+  klass = POLKIT_BACKEND_AUTHORITY_GET_CLASS (authority);
+
+  if (klass->remove_lockdown_for_action_finish == NULL)
+    {
+      g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+      return FALSE;
+    }
+  else
+    {
+      return klass->remove_lockdown_for_action_finish (authority, res, error);
+    }
+}
 
 /* ---------------------------------------------------------------------------------------------------- */
 
@@ -483,6 +684,14 @@ struct _ServerClass
   GObjectClass parent_class;
 };
 
+enum
+{
+  PROP_0,
+  PROP_BACKEND_NAME,
+  PROP_BACKEND_VERSION,
+  PROP_BACKEND_FEATURES
+};
+
 static void authority_iface_init         (_PolkitAuthorityIface        *authority_iface);
 
 G_DEFINE_TYPE_WITH_CODE (Server, server, G_TYPE_OBJECT,
@@ -520,6 +729,35 @@ server_finalize (GObject *object)
   g_hash_table_unref (server->cancellation_id_to_cancellable);
 }
 
+static void
+server_get_property (GObject    *object,
+                     guint       prop_id,
+                     GValue     *value,
+                     GParamSpec *pspec)
+{
+  Server *server = SERVER (object);
+
+  switch (prop_id)
+    {
+    case PROP_BACKEND_NAME:
+      g_value_set_string (value, polkit_backend_authority_get_name (server->authority));
+      break;
+
+    case PROP_BACKEND_VERSION:
+      g_value_set_string (value, polkit_backend_authority_get_version (server->authority));
+      break;
+
+    case PROP_BACKEND_FEATURES:
+      g_value_set_flags (value, polkit_backend_authority_get_features (server->authority));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
 static void
 server_class_init (ServerClass *klass)
 {
@@ -528,6 +766,9 @@ server_class_init (ServerClass *klass)
   gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = server_finalize;
+  gobject_class->get_property = server_get_property;
+
+  g_assert (_polkit_authority_override_properties (gobject_class, PROP_BACKEND_NAME) == PROP_BACKEND_FEATURES);
 }
 
 static void
@@ -658,10 +899,20 @@ authority_handle_check_authorization (_PolkitAuthority               *instance,
   GCancellable *cancellable;
   PolkitDetails *details;
 
-  caller_name = egg_dbus_method_invocation_get_caller (method_invocation);
-  caller = polkit_system_bus_name_new (caller_name);
+  details = NULL;
 
   subject = polkit_subject_new_for_real (real_subject);
+  if (subject == NULL)
+    {
+      egg_dbus_method_invocation_return_error_literal (method_invocation,
+                                                       _POLKIT_ERROR,
+                                                       _POLKIT_ERROR_FAILED,
+                                                       "Error parsing subject struct");
+      goto out;
+    }
+
+  caller_name = egg_dbus_method_invocation_get_caller (method_invocation);
+  caller = polkit_system_bus_name_new (caller_name);
 
   details = polkit_details_new_for_hash (real_details->data);
 
@@ -707,7 +958,8 @@ authority_handle_check_authorization (_PolkitAuthority               *instance,
                                                 check_auth_cb,
                                                 method_invocation);
  out:
-  g_object_unref (details);
+  if (details != NULL)
+    g_object_unref (details);
 }
 
 static void
@@ -758,10 +1010,21 @@ authority_handle_register_authentication_agent (_PolkitAuthority               *
   PolkitSubject *subject;
   GError *error;
 
-  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
+  caller = NULL;
+
   subject = polkit_subject_new_for_real (real_subject);
+  if (subject == NULL)
+    {
+      egg_dbus_method_invocation_return_error_literal (method_invocation,
+                                                       _POLKIT_ERROR,
+                                                       _POLKIT_ERROR_FAILED,
+                                                       "Error parsing subject struct");
+      goto out;
+    }
   g_object_set_data_full (G_OBJECT (method_invocation), "subject", subject, (GDestroyNotify) g_object_unref);
 
+  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
+
   error = NULL;
   if (!polkit_backend_authority_register_authentication_agent (server->authority,
                                                                caller,
@@ -778,7 +1041,8 @@ authority_handle_register_authentication_agent (_PolkitAuthority               *
   _polkit_authority_handle_register_authentication_agent_finish (method_invocation);
 
  out:
-  g_object_unref (caller);
+  if (caller != NULL)
+    g_object_unref (caller);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -794,10 +1058,21 @@ authority_handle_unregister_authentication_agent (_PolkitAuthority
   PolkitSubject *subject;
   GError *error;
 
-  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
+  caller = NULL;
+
   subject = polkit_subject_new_for_real (real_subject);
+  if (subject == NULL)
+    {
+      egg_dbus_method_invocation_return_error_literal (method_invocation,
+                                                       _POLKIT_ERROR,
+                                                       _POLKIT_ERROR_FAILED,
+                                                       "Error parsing subject struct");
+      goto out;
+    }
   g_object_set_data_full (G_OBJECT (method_invocation), "subject", subject, (GDestroyNotify) g_object_unref);
 
+  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
+
   error = NULL;
   if (!polkit_backend_authority_unregister_authentication_agent (server->authority,
                                                                  caller,
@@ -813,7 +1088,8 @@ authority_handle_unregister_authentication_agent (_PolkitAuthority
   _polkit_authority_handle_unregister_authentication_agent_finish (method_invocation);
 
  out:
-  g_object_unref (caller);
+  if (caller != NULL)
+    g_object_unref (caller);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -829,7 +1105,18 @@ authority_handle_authentication_agent_response (_PolkitAuthority               *
   PolkitIdentity *identity;
   GError *error;
 
+  caller = NULL;
+  identity = NULL;
+
   identity = polkit_identity_new_for_real (real_identity);
+  if (identity == NULL)
+    {
+      egg_dbus_method_invocation_return_error_literal (method_invocation,
+                                                       _POLKIT_ERROR,
+                                                       _POLKIT_ERROR_FAILED,
+                                                       "Error parsing identity struct");
+      goto out;
+    }
 
   caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
 
@@ -848,9 +1135,11 @@ authority_handle_authentication_agent_response (_PolkitAuthority               *
   _polkit_authority_handle_authentication_agent_response_finish (method_invocation);
 
  out:
-  g_object_unref (caller);
+  if (caller != NULL)
+    g_object_unref (caller);
 
-  g_object_unref (identity);
+  if (identity != NULL)
+    g_object_unref (identity);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -872,11 +1161,19 @@ authority_handle_enumerate_temporary_authorizations (_PolkitAuthority        *in
   caller = NULL;
   temporary_authorizations = NULL;
 
-  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
-
   subject = polkit_subject_new_for_real (real_subject);
+  if (subject == NULL)
+    {
+      egg_dbus_method_invocation_return_error_literal (method_invocation,
+                                                       _POLKIT_ERROR,
+                                                       _POLKIT_ERROR_FAILED,
+                                                       "Error parsing subject struct");
+      goto out;
+    }
   g_object_set_data_full (G_OBJECT (method_invocation), "subject", subject, (GDestroyNotify) g_object_unref);
 
+  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
+
   temporary_authorizations = polkit_backend_authority_enumerate_temporary_authorizations (server->authority,
                                                                                           caller,
                                                                                           subject,
@@ -909,7 +1206,8 @@ authority_handle_enumerate_temporary_authorizations (_PolkitAuthority        *in
  out:
   g_list_foreach (temporary_authorizations, (GFunc) g_object_unref, NULL);
   g_list_free (temporary_authorizations);
-  g_object_unref (caller);
+  if (caller != NULL)
+    g_object_unref (caller);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -927,11 +1225,19 @@ authority_handle_revoke_temporary_authorizations (_PolkitAuthority        *insta
   error = NULL;
   caller = NULL;
 
-  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
-
   subject = polkit_subject_new_for_real (real_subject);
+  if (subject == NULL)
+    {
+      egg_dbus_method_invocation_return_error_literal (method_invocation,
+                                                       _POLKIT_ERROR,
+                                                       _POLKIT_ERROR_FAILED,
+                                                       "Error parsing subject struct");
+      goto out;
+    }
   g_object_set_data_full (G_OBJECT (method_invocation), "subject", subject, (GDestroyNotify) g_object_unref);
 
+  caller = polkit_system_bus_name_new (egg_dbus_method_invocation_get_caller (method_invocation));
+
   polkit_backend_authority_revoke_temporary_authorizations (server->authority,
                                                             caller,
                                                             subject,
@@ -946,7 +1252,8 @@ authority_handle_revoke_temporary_authorizations (_PolkitAuthority        *insta
   _polkit_authority_handle_revoke_temporary_authorizations_finish (method_invocation);
 
  out:
-  g_object_unref (caller);
+  if (caller != NULL)
+    g_object_unref (caller);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -983,6 +1290,96 @@ authority_handle_revoke_temporary_authorization_by_id (_PolkitAuthority        *
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+add_lockdown_cb (GObject      *source_object,
+                 GAsyncResult *res,
+                 gpointer      user_data)
+{
+  EggDBusMethodInvocation *method_invocation = EGG_DBUS_METHOD_INVOCATION (user_data);
+  GError *error;
+
+  error = NULL;
+  polkit_backend_authority_add_lockdown_for_action_finish (POLKIT_BACKEND_AUTHORITY (source_object),
+                                                           res,
+                                                           &error);
+
+  if (error != NULL)
+    {
+      egg_dbus_method_invocation_return_gerror (method_invocation, error);
+      g_error_free (error);
+    }
+  else
+    {
+      _polkit_authority_handle_add_lockdown_for_action_finish (method_invocation);
+    }
+}
+
+static void
+authority_handle_add_lockdown_for_action (_PolkitAuthority               *instance,
+                                          const gchar                    *action_id,
+                                          EggDBusMethodInvocation        *method_invocation)
+{
+  Server *server = SERVER (instance);
+  const gchar *caller_name;
+  PolkitSubject *caller;
+
+  caller_name = egg_dbus_method_invocation_get_caller (method_invocation);
+  caller = polkit_system_bus_name_new (caller_name);
+
+  polkit_backend_authority_add_lockdown_for_action (server->authority,
+                                                    caller,
+                                                    action_id,
+                                                    add_lockdown_cb,
+                                                    method_invocation);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static void
+remove_lockdown_cb (GObject      *source_object,
+                    GAsyncResult *res,
+                    gpointer      user_data)
+{
+  EggDBusMethodInvocation *method_invocation = EGG_DBUS_METHOD_INVOCATION (user_data);
+  GError *error;
+
+  error = NULL;
+  polkit_backend_authority_remove_lockdown_for_action_finish (POLKIT_BACKEND_AUTHORITY (source_object),
+                                                              res,
+                                                              &error);
+
+  if (error != NULL)
+    {
+      egg_dbus_method_invocation_return_gerror (method_invocation, error);
+      g_error_free (error);
+    }
+  else
+    {
+      _polkit_authority_handle_remove_lockdown_for_action_finish (method_invocation);
+    }
+}
+
+static void
+authority_handle_remove_lockdown_for_action (_PolkitAuthority               *instance,
+                                             const gchar                    *action_id,
+                                             EggDBusMethodInvocation        *method_invocation)
+{
+  Server *server = SERVER (instance);
+  const gchar *caller_name;
+  PolkitSubject *caller;
+
+  caller_name = egg_dbus_method_invocation_get_caller (method_invocation);
+  caller = polkit_system_bus_name_new (caller_name);
+
+  polkit_backend_authority_remove_lockdown_for_action (server->authority,
+                                                       caller,
+                                                       action_id,
+                                                       remove_lockdown_cb,
+                                                       method_invocation);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static void
 authority_iface_init (_PolkitAuthorityIface *authority_iface)
 {
@@ -995,6 +1392,8 @@ authority_iface_init (_PolkitAuthorityIface *authority_iface)
   authority_iface->handle_enumerate_temporary_authorizations   = authority_handle_enumerate_temporary_authorizations;
   authority_iface->handle_revoke_temporary_authorizations      = authority_handle_revoke_temporary_authorizations;
   authority_iface->handle_revoke_temporary_authorization_by_id = authority_handle_revoke_temporary_authorization_by_id;
+  authority_iface->handle_add_lockdown_for_action              = authority_handle_add_lockdown_for_action;
+  authority_iface->handle_remove_lockdown_for_action              = authority_handle_remove_lockdown_for_action;
 }
 
 static void
